@@ -1,17 +1,21 @@
+import os
 import csv
 import json
+import itertools
 import math
 import pytz
 import shutil
 import six
 import tempfile
+import time
 
 from collections import defaultdict
 from datetime import datetime
 from io import StringIO, BytesIO
 
-from flask import Flask, jsonify
+from flask import Flask
 from structlog import get_logger
+from werkzeug.http import http_date
 
 from .config import configure
 from .encoders import JSONEncoder
@@ -19,11 +23,26 @@ from .render.attime import parseATTime
 from .render.datalib import fetchData, TimeSeries
 from .render.glyph import GraphTypes
 from .render.grammar import grammar
-from .utils import RequestParams
+from .utils import RequestParams, hash_request
 
 logger = get_logger()
 
 
+def jsonify(data, status=200, headers=None):
+    if headers is None:
+        headers = {}
+
+    jsonp = RequestParams.get('jsonp', False)
+
+    body = json.dumps(data, cls=JSONEncoder)
+    if jsonp:
+        headers['Content-Type'] = 'text/javascript'
+        body = '{0}({1})'.format(jsonp, body)
+    else:
+        headers['Content-Type'] = 'application/json'
+    return body, status, headers
+
+
 class Graphite(Flask):
     @property
     def store(self):
@@ -45,7 +64,12 @@ class Graphite(Flask):
 
 
 app = Graphite(__name__)
-configure(app)
+try:
+    configure(app)
+except Exception:
+    import traceback
+    print(traceback.format_exc())
+    raise
 
 methods = ('GET', 'POST')
 
@@ -58,8 +82,8 @@ def dashboard_find():
 
 @app.route('/dashboard/load/<name>', methods=methods)
 def dashboard_load(name):
-    return jsonify(
-        {'error': "Dashboard '{0}' does not exist.".format(name)}), 404
+    return jsonify({'error': "Dashboard '{0}' does not exist.".format(name)},
+                   status=404)
 
 
 @app.route('/events/get_data', methods=methods)
@@ -68,24 +92,7 @@ def events():
 
 
 # API calls that actually do something
-@app.route('/metrics/search', methods=methods)
-def metrics_search():
-    errors = {}
-    try:
-        max_results = int(RequestParams.get('max_results', 25))
-    except ValueError:
-        errors['max_results'] = 'must be an integer.'
-    if not 'query' in RequestParams:
-        errors['query'] = 'this parameter is required.'
-    if errors:
-        return jsonify({'errors': errors}), 400
-    results = sorted(app.searcher.search(
-        query=RequestParams['query'],
-        max_results=max_results,
-    ), key=lambda result: result['path'] or '')
-    return jsonify({'metrics': results})
-
-
+@app.route('/metrics', methods=methods)
 @app.route('/metrics/find', methods=methods)
 def metrics_find():
     errors = {}
@@ -99,23 +106,28 @@ def metrics_find():
         errors['wildcards'] = 'must be 0 or 1.'
 
     try:
-        from_time = int(RequestParams.get('from', 0))
+        from_time = int(RequestParams.get('from', -1))
     except ValueError:
         errors['from'] = 'must be an epoch timestamp.'
     try:
-        until_time = int(RequestParams.get('until', 0))
+        until_time = int(RequestParams.get('until', -1))
     except ValueError:
         errors['until'] = 'must be an epoch timestamp.'
 
+    if from_time == -1:
+        from_time = None
+    if until_time == -1:
+        until_time = None
+
     format = RequestParams.get('format', 'treejson')
-    if not format in ['treejson', 'completer']:
+    if format not in ['treejson', 'completer']:
         errors['format'] = 'unrecognized format: "{0}".'.format(format)
 
-    if not 'query' in RequestParams:
+    if 'query' not in RequestParams:
         errors['query'] = 'this parameter is required.'
 
     if errors:
-        return jsonify({'errors': errors}), 400
+        return jsonify({'errors': errors}, status=400)
 
     query = RequestParams['query']
     matches = sorted(
@@ -162,10 +174,10 @@ def metrics_expand():
     except ValueError:
         errors['leavesOnly'] = 'must be 0 or 1.'
 
-    if not 'query' in RequestParams:
+    if 'query' not in RequestParams:
         errors['query'] = 'this parameter is required.'
     if errors:
-        return jsonify({'errors': errors}), 400
+        return jsonify({'errors': errors}, status=400)
 
     results = defaultdict(set)
     for query in RequestParams.getlist('query'):
@@ -185,6 +197,19 @@ def metrics_expand():
     return jsonify({'results': results})
 
 
+@app.route('/metrics/index.json', methods=methods)
+def metrics_index():
+    index = set()
+    if os.path.exists(app.searcher.index_path):
+        with open(app.searcher.index_path, 'r') as f:
+            index = set([line.strip() for line in f if line])
+        if not index:
+            recurse('*', index)
+    else:
+        recurse('*', index)
+    return jsonify(sorted(index))
+
+
 def prune_datapoints(series, max_datapoints, start, end):
     time_range = end - start
     points = time_range // series.step
@@ -194,9 +219,9 @@ def prune_datapoints(series, max_datapoints, start, end):
         )
         seconds_per_point = values_per_point * series.step
         nudge = (
-            seconds_per_point
-            + (series.start % series.step)
-            - (series.start % seconds_per_point)
+            seconds_per_point +
+            (series.start % series.step) -
+            (series.start % seconds_per_point)
         )
         series.start += nudge
         values_to_lose = nudge // series.step
@@ -206,7 +231,7 @@ def prune_datapoints(series, max_datapoints, start, end):
     else:
         step = series.step
 
-    timestamps = range(series.start, series.end, step)
+    timestamps = range(series.start, series.end + series.step, step)
     datapoints = zip(series, timestamps)
     return {'target': series.name, 'datapoints': datapoints}
 
@@ -230,7 +255,7 @@ def build_index():
         index_file.write('\n'.join(sorted(index)).encode('utf-8'))
     shutil.move(index_file.name, app.searcher.index_path)
     app.searcher.reload()
-    return jsonify({'success': True, 'entries': len(index)}), 200
+    return jsonify({'success': True, 'entries': len(index)})
 
 
 @app.route('/render', methods=methods)
@@ -270,7 +295,7 @@ def render():
             errors['maxDataPoints'] = 'Must be an integer.'
 
     if errors:
-        return jsonify({'errors': errors}), 400
+        return jsonify({'errors': errors}, status=400)
 
     for opt in graph_class.customizable:
         if opt in RequestParams:
@@ -307,12 +332,27 @@ def render():
     request_options['startTime'] = start_time
     request_options['endTime'] = end_time
 
+    use_cache = app.cache is not None and 'noCache' not in RequestParams
+    cache_timeout = RequestParams.get('cacheTimeout')
+    if cache_timeout is not None:
+        cache_timeout = int(cache_timeout)
+
     if errors:
-        return jsonify({'errors': errors}), 400
+        return jsonify({'errors': errors}, status=400)
 
     # Done with options.
 
+    if use_cache:
+        request_key = hash_request()
+        response = app.cache.get(request_key)
+        if response is not None:
+            return response
+
     headers = {
+        'Last-Modified': http_date(time.time()),
+        'Expires': http_date(time.time() + (cache_timeout or 60)),
+        'Cache-Control': 'max-age={0}'.format(cache_timeout or 60)
+    } if use_cache else {
         'Pragma': 'no-cache',
         'Cache-Control': 'no-cache',
     }
@@ -320,9 +360,20 @@ def render():
     context = {
         'startTime': request_options['startTime'],
         'endTime': request_options['endTime'],
+        'tzinfo': request_options['tzinfo'],
         'data': [],
     }
 
+    # Gather all data to take advantage of backends with fetch_multi
+    paths = []
+    for target in request_options['targets']:
+        if request_options['graphType'] == 'pie':
+            if ':' in target:
+                continue
+        if target.strip():
+            paths += pathsFromTarget(target)
+    data_store = fetchData(context, paths)
+
     if request_options['graphType'] == 'pie':
         for target in request_options['targets']:
             if ':' in target:
@@ -333,7 +384,7 @@ def render():
                     errors['target'] = "Invalid target: '{0}'.".format(target)
                 context['data'].append((name, value))
             else:
-                series_list = evaluateTarget(context, target)
+                series_list = evaluateTarget(context, target, data_store)
 
                 for series in series_list:
                     func = app.functions[request_options['pieMode']]
@@ -341,13 +392,13 @@ def render():
                                             func(context, series) or 0))
 
         if errors:
-            return jsonify({'errors': errors}), 400
+            return jsonify({'errors': errors}, status=400)
 
     else:  # graphType == 'line'
         for target in request_options['targets']:
             if not target.strip():
                 continue
-            series_list = evaluateTarget(context, target)
+            series_list = evaluateTarget(context, target, data_store)
             context['data'].extend(series_list)
 
         request_options['format'] = request_options.get('format')
@@ -384,23 +435,14 @@ def render():
                     series_data.append({'target': series.name,
                                         'datapoints': datapoints})
 
-            rendered = json.dumps(series_data, cls=JSONEncoder)
-
-            if 'jsonp' in request_options:
-                headers['Content-Type'] = 'text/javascript'
-                return ('{0}({1})'.format(request_options['jsonp'],
-                                          rendered), 200,
-                        headers)
-            else:
-                headers['Content-Type'] = 'application/json'
-                return rendered, 200, headers
+            return jsonify(series_data, headers=headers)
 
         if request_options['format'] == 'raw':
             response = StringIO()
             for series in context['data']:
                 response.write(u"%s,%d,%d,%d|" % (
                     series.name, series.start, series.end, series.step))
-                response.write(u','.join(map(str, series)))
+                response.write(u','.join(map(repr, series)))
                 response.write(u'\n')
             response.seek(0)
             headers['Content-Type'] = 'text/plain'
@@ -416,18 +458,42 @@ def render():
 
     if use_svg and 'jsonp' in request_options:
         headers['Content-Type'] = 'text/javascript'
-        return ('{0}({1})'.format(request_options['jsonp'],
-                                  json.dumps(image.decode('utf-8'))),
-                200, headers)
+        response = ('{0}({1})'.format(request_options['jsonp'],
+                                      json.dumps(image.decode('utf-8'))),
+                    200, headers)
     else:
         ctype = 'image/svg+xml' if use_svg else 'image/png'
         headers['Content-Type'] = ctype
-        return image, 200, headers
+        response = image, 200, headers
+
+    if use_cache:
+        app.cache.add(request_key, response, cache_timeout)
+    return response
 
 
-def evaluateTarget(requestContext, target):
+def pathsFromTarget(target):
     tokens = grammar.parseString(target)
-    result = evaluateTokens(requestContext, tokens)
+    return list(pathsFromTokens(tokens))
+
+
+def pathsFromTokens(tokens):
+    iters = []
+    if tokens.expression:
+        iters.append(pathsFromTokens(tokens.expression))
+    elif tokens.pathExpression:
+        iters.append([tokens.pathExpression])
+    elif tokens.call:
+        iters.extend([pathsFromTokens(arg)
+                      for arg in tokens.call.args])
+        iters.extend([pathsFromTokens(kwarg.args[0])
+                      for kwarg in tokens.call.kwargs])
+    for path in itertools.chain(*iters):
+        yield(path)
+
+
+def evaluateTarget(requestContext, target, data_store):
+    tokens = grammar.parseString(target)
+    result = evaluateTokens(requestContext, tokens, data_store)
 
     if isinstance(result, TimeSeries):
         return [result]  # we have to return a list of TimeSeries objects
@@ -435,21 +501,24 @@ def evaluateTarget(requestContext, target):
     return result
 
 
-def evaluateTokens(requestContext, tokens):
+def evaluateTokens(requestContext, tokens, data_store):
     if tokens.expression:
-        return evaluateTokens(requestContext, tokens.expression)
+        return evaluateTokens(requestContext, tokens.expression, data_store)
 
     elif tokens.pathExpression:
-        return fetchData(requestContext, tokens.pathExpression)
+        return data_store.get_series_list(tokens.pathExpression)
 
     elif tokens.call:
         func = app.functions[tokens.call.funcname]
         args = [evaluateTokens(requestContext,
-                               arg) for arg in tokens.call.args]
+                               arg, data_store) for arg in tokens.call.args]
         kwargs = dict([(kwarg.argname,
-                        evaluateTokens(requestContext, kwarg.args[0]))
+                        evaluateTokens(requestContext,
+                                       kwarg.args[0],
+                                       data_store))
                        for kwarg in tokens.call.kwargs])
-        return func(requestContext, *args, **kwargs)
+        ret = func(requestContext, *args, **kwargs)
+        return ret
 
     elif tokens.number:
         if tokens.number.integer:
